@@ -1,10 +1,10 @@
-# Example for: alignment.read_one()
+# Example for: Alignment.read_one()
 
 from modeller import *
-env = environ()
+env = Environ()
 
 # Create an empty alignment
-aln = alignment(env)
+aln = Alignment(env)
 
 # Open the input alignment file, and get a handle to it:
 input = modfile.File('toxin.ali', 'r')
