@@ -4,9 +4,25 @@
 
 import modeller.features
 import modeller.forms
+import modeller.coordinates
 import _modeller
 import string
 
+def _get_metric_value(s):
+    value = s.value
+    if isinstance(value, (tuple, list)):
+        value = value[0]
+    return value
+
+
+def _update_entity_type(entity_type, new_entity_type):
+    if entity_type is None:
+        entity_type = new_entity_type
+    # We selected polymer AND non-polymer or water
+    elif entity_type != new_entity_type:
+        entity_type = 'other'
+    return entity_type
+
 
 class _AsymIDs(object):
     """Map indices to multi-character asym (chain) IDs.
@@ -120,12 +136,51 @@ class DihedralRestraints(Restraints, list):
     category = '_ma_dihedral_restraints'
 
 
+class _Asym(object):
+    """A single mmCIF asym. This does not map 1:1 to Modeller chains, as
+       polymers and ligands must be in different mmCIF asyms."""
+    def __init__(self, asym_from_int_id, mdl, iasym):
+        self.mdl = mdl
+        self._asym_from_int_id = asym_from_int_id
+        self._iasym = iasym
+        (self.entity_id, self.resbeg, self.resend, self.polymer,
+         self.water) = _modeller.mod_model_get_asym(mdl.modpt, iasym)
+
+    def get_atom_indices(self):
+        startatm, endatm = modeller.coordinates.get_residue_atom_indices(
+            self.mdl, self.resbeg, self.resend + 1)
+        return (range(startatm+1, endatm+1), self.mdl)
+
+    def __lt__(self, other):
+        return self.resbeg < other.resbeg
+
+    def __get_entity_type(self):
+        if self.polymer:
+            return 'polymer'
+        elif self.water:
+            return 'water'
+        else:
+            return 'non-polymer'
+
+    asym_id = property(lambda self: self._asym_from_int_id[self._iasym])
+    entity_type = property(__get_entity_type)
+
+
 class _AsymMap(object):
     def __init__(self, mdl):
         self.mdl = mdl
         self._asym_from_int_id = _AsymIDs()
         _modeller.mod_model_make_mmcif_asyms(mdl.modpt, mdl.env.libs.modpt)
 
+    def get_all_asyms(self):
+        """Get information on all mmCIF asyms, sorted by atom index"""
+        nasym = _modeller.mod_model_get_num_asyms(self.mdl.modpt)
+        asyms = []
+        for i in range(nasym):
+            asyms.append(_Asym(self._asym_from_int_id, self.mdl, i))
+        asyms.sort()
+        return asyms
+
     def map_residue(self, res):
         """Map a Residue object to asym, entity, sequence IDs"""
         int_asym_id, entity_id, seq_id = _modeller.mod_model_get_residue_asym(
@@ -145,17 +200,23 @@ class _AsymMap(object):
 
 
 class CifData(object):
+    _feature_details = 'Selected model subset'
+
     def __init__(self, mdl):
         self.data = []
         self.mdl = mdl
         _modeller.mod_model_clear_mmcif_file_data(self.mdl.modpt)
         self._asym_map = _AsymMap(mdl)
+        self._qa_scores = []
 
     def add_alignment_info(self, aln, knowns, seq):
         self._add_target_chains(aln[seq], self.mdl)
         self._add_template_chains(aln, knowns, aln[seq])
         self._add_alignments(aln)
 
+    def add_qa_scores(self, qa_scores):
+        self._qa_scores.extend(qa_scores)
+
     def add_restraints(self, user_restraints):
         self.restraints = []
         # Only handle single-feature Gaussian-type restraints for now
@@ -286,6 +347,7 @@ class CifData(object):
             ordinal += 1
 
     def write_mmcif(self, writer):
+        """Write out all information for regular comparative modeling"""
         target_seq = TargetData()
         target_seq.name = 'Target Sequence'
         self._add_data(target_seq, True)
@@ -297,6 +359,11 @@ class CifData(object):
         self._write_poly_mapping(writer)
         self._write_alignment(writer)
         self._write_restraints(writer)
+        self._write_qa_scores(writer, self.mdl.select_atoms)
+
+    def write_loop_mmcif(self, writer):
+        """Write out all information for loop refinement"""
+        self._write_qa_scores(writer, self.mdl.select_loop_atoms)
 
     def _write_template_details(self, writer):
         with writer.loop(
@@ -414,6 +481,158 @@ class CifData(object):
         for r in self.restraints:
             self._write_type_restraints(writer, r)
 
+    def _write_qa_scores(self, writer, select_func):
+        mode_map = {'model': 'global', 'selection': 'per-feature'}
+        for s in self._qa_scores:
+            if s.target == 'selection':
+                mode_map['selection'] = self._write_selection_feature(
+                    writer, select_func)
+                break
+        self._write_qa_metric_table(writer, mode_map)
+        self._write_qa_global(writer, mode_map)
+        self._write_qa_feature(writer, mode_map)
+
+    def _write_selection_feature(self, writer, select_func):
+        inds, m = select_func().get_atom_indices()
+        # If the entire model is selected, we don't need to write a feature;
+        # instead, report all QA metrics as 'global'
+        if len(inds) == len(self.mdl.atoms):
+            return 'global'
+        asyms = self._asym_map.get_all_asyms()
+        if (not self._try_write_instance_feature(writer, inds, asyms)
+                and not self._try_write_poly_residue_feature(writer, inds)):
+            self._write_atom_feature(writer, inds, asyms)
+        return 'per-feature'
+
+    def _try_write_instance_feature(self, writer, inds, asyms):
+        """Iff the model selection selects entire mmCIF asyms, output the
+           ma_entity_instance_feature table and return True"""
+        atom = 0
+        selected_asyms = []
+        entity_type = None
+        for asym in asyms:
+            asym_inds, _ = asym.get_atom_indices()
+            len_asym = len(asym_inds)
+            if len_asym == 0:
+                continue
+            # Atom is partway through a previous asym, so we are not selecting
+            # an entire asym
+            if inds[atom] < asym_inds[0]:
+                return False
+            # This asym is not selected
+            elif inds[atom] > asym_inds[0]:
+                continue
+            # Start asym atom is selected; check that the end asym atom is
+            # selected too
+            elif (atom + len_asym > len(inds)
+                  or inds[atom + len_asym - 1] != asym_inds[-1]):
+                return False
+            else:
+                entity_type = _update_entity_type(entity_type, asym.entity_type)
+                selected_asyms.append(asym)
+                atom += len_asym
+                if atom >= len(inds):
+                    break
+        # Ran out of asyms without checking all atoms
+        if atom < len(inds):
+            return False
+        self._write_ma_feature_list(writer, 'entity instance', entity_type)
+        with writer.loop('_ma_entity_instance_feature',
+                         ['ordinal_id', 'feature_id', 'label_asym_id']) as lp:
+            for i, asym in enumerate(selected_asyms):
+                lp.write(ordinal_id=i + 1, feature_id=1,
+                         label_asym_id=asym.asym_id)
+        return True
+
+    def _try_write_poly_residue_feature(self, writer, inds):
+        atom = 0
+        selected_residues = []
+        while atom < len(inds):
+            res = self.mdl.atoms[inds[atom] - 1].residue
+            natms = len(res.atoms)
+            # If the entire residue is selected, atom indices must be sequential
+            if (atom + natms > len(inds)
+                    or inds[atom + natms - 1] != inds[atom] + natms - 1):
+                return False
+            else:
+                atom += natms
+                asym_id, entity_id, seq_id = self._asym_map.map_residue(res)
+                # Only polymeric residues can use this table
+                if seq_id <= 0:
+                    return False
+                selected_residues.append((res, asym_id, seq_id))
+        self._write_ma_feature_list(writer, 'residue', 'polymer')
+        with writer.loop('_ma_poly_residue_feature',
+                         ['ordinal_id', 'feature_id', 'label_asym_id',
+                          'label_seq_id', 'label_comp_id']) as lp:
+            for i, sr in enumerate(selected_residues):
+                r, asym_id, seq_id = sr
+                lp.write(ordinal_id=i + 1, feature_id=1,
+                         label_asym_id=asym_id, label_seq_id=seq_id,
+                         label_comp_id=r.name)
+        return True
+
+    def _write_atom_feature(self, writer, inds, asyms):
+        iasym = -1
+        entity_type = None
+        for ind in inds:
+            while iasym < 0 or asym_inds[0] > ind or asym_inds[-1] < ind:
+                iasym += 1
+                asym_inds, _ = asyms[iasym].get_atom_indices()
+            entity_type = _update_entity_type(entity_type,
+                                              asyms[iasym].entity_type)
+        self._write_ma_feature_list(writer, 'atom', entity_type)
+        with writer.loop('_ma_atom_feature',
+                         ['ordinal_id', 'feature_id', 'atom_id']) as lp:
+            for i, ind in enumerate(inds):
+                # atom_id is the same as the Modeller internal atom index
+                # since mmCIF does not use TER records
+                lp.write(ordinal_id=i + 1, feature_id=1,
+                         atom_id=ind)
+
+    def _write_ma_feature_list(self, writer, feature_type, entity_type):
+        with writer.loop('_ma_feature_list',
+                         ['feature_id', 'feature_type', 'entity_type',
+                          'details']) as lp:
+            lp.write(feature_id=1, feature_type=feature_type,
+                     entity_type=entity_type,
+                     details=self._feature_details)
+
+    def _write_qa_metric_table(self, writer, mode_map):
+        with writer.loop('_ma_qa_metric',
+                         ['id', 'name', 'description', 'type', 'mode',
+                          'type_other_details', 'software_group_id']) as lp:
+            for i, s in enumerate(self._qa_scores):
+                lp.write(id=i + 1, name=s.name,
+                         description=s.description, type=s.metric_type,
+                         mode=mode_map[s.target],
+                         # We use group 1 internally for Modeller itself
+                         software_group_id=1)
+
+    def _write_qa_global(self, writer, mode_map):
+        with writer.loop('_ma_qa_metric_global',
+                         ['ordinal_id', 'model_id', 'metric_id',
+                          'metric_value']) as lp:
+            ordinal = 1
+            for i, s in enumerate(self._qa_scores):
+                if mode_map[s.target] != 'global':
+                    continue
+                lp.write(ordinal_id=ordinal, model_id=1, metric_id=i + 1,
+                         metric_value=_get_metric_value(s))
+                ordinal += 1
+
+    def _write_qa_feature(self, writer, mode_map):
+        with writer.loop('_ma_qa_metric_feature',
+                         ['ordinal_id', 'model_id', 'feature_id', 'metric_id',
+                          'metric_value']) as lp:
+            ordinal = 1
+            for i, s in enumerate(self._qa_scores):
+                if mode_map[s.target] != 'per-feature':
+                    continue
+                lp.write(ordinal_id=ordinal, model_id=1, feature_id=1,
+                         metric_id=i + 1, metric_value=_get_metric_value(s))
+                ordinal += 1
+
     def _get_rsr_limits(self, r):
         """Get the limits for a given restraint"""
         mean, stddev = r._parameters[0:2]
