@@ -731,9 +731,10 @@ class Selection(object):
     def assess(self, assessor, output='SHORT NO_REPORT', **vars):
         """Assess with the given assessor object
            (e.g. :class:`soap_loop.Scorer`)."""
-        print(">> Model assessment by %s" % assessor.name)
+        name = assessor._Metric.name
+        print(">> Model assessment by %s" % name)
         molpdf, terms = assessor._assess(self, output=output, **vars)
-        print("%s                     : %12.6f" % (assessor.name, molpdf))
+        print("%s                     : %12.6f" % (name, molpdf))
         return molpdf
 
     def _dope_energy(self, gprsr, name, output='SHORT NO_REPORT',
