@@ -9,7 +9,10 @@ __docformat__ = "epytext en"
 class Scorer(terms.AssessEnergyTerm):
     """Score the model using the SOAP-Loop potential."""
 
-    name = "SOAP-Loop score"
+    class _Metric(terms.SelectionQAMetric):
+        metric_type = 'energy'
+        name = "SOAP-Loop score"
+        description = 'SOAP for scoring loop conformations'
 
     def __init__(self, library='${LIB}/soap_loop.hdf5',
                  group=physical.xy_distance):
